# High-Level Brainfuck
#
# Copyright (c) 2004 Marek Materzok
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

import parser
import string


# Begin -- grammar generated by Yapps
import sys, re
from yapps import runtime

class HLBFScanner(runtime.Scanner):
    patterns = [
        ('"\\$"', re.compile('\\$')),
        ('"in"', re.compile('in')),
        ('"%"', re.compile('%')),
        ('"/"', re.compile('/')),
        ('"\\*"', re.compile('\\*')),
        ('"-"', re.compile('-')),
        ('"\\+"', re.compile('\\+')),
        ('"<="', re.compile('<=')),
        ('"<"', re.compile('<')),
        ('">="', re.compile('>=')),
        ('">"', re.compile('>')),
        ('"!="', re.compile('!=')),
        ('"="', re.compile('=')),
        ('"&&"', re.compile('&&')),
        ('"||"', re.compile('||')),
        ('"\\""', re.compile('"')),
        ('"null"', re.compile('null')),
        ('"while"', re.compile('while')),
        ('"else"', re.compile('else')),
        ('"if"', re.compile('if')),
        ('"<-"', re.compile('<-')),
        ('"out"', re.compile('out')),
        ('"def"', re.compile('def')),
        ('"return"', re.compile('return')),
        ('"do"', re.compile('do')),
        ('"}"', re.compile('}')),
        ('";"', re.compile(';')),
        ('"{"', re.compile('{')),
        ('"of"', re.compile('of')),
        ('"\\]"', re.compile('\\]')),
        ('"\\["', re.compile('\\[')),
        ('"array"', re.compile('array')),
        ('"byte"', re.compile('byte')),
        ('"void"', re.compile('void')),
        ('"\\)"', re.compile('\\)')),
        ('","', re.compile(',')),
        ('"&"', re.compile('&')),
        ('"\\("', re.compile('\\(')),
        ('\\s+', re.compile('\\s+')),
        ('NUM', re.compile('[0-9]+')),
        ('ID', re.compile('[a-zA-Z][a-zA-Z0-9_]*')),
        ('END', re.compile('$')),
        ('TEXT', re.compile('([a-zA-Z0-9!?., ]|\\\\([ntr]|x[0-9a-fA-F][0-9a-fA-F]))*')),
    ]
    def __init__(self, str,*args,**kw):
        runtime.Scanner.__init__(self,None,{'\\s+':None,},str,*args,**kw)

class HLBF(runtime.Parser):
    Context = runtime.Context
    def program(self, _parent=None):
        _context = self.Context(_parent, self._scanner, 'program', [])
        toplevel = []
        while self._peek('END', '"void"', '"byte"', '"array"', context=_context) != 'END':
            function = self.function(_context)
            toplevel.append(function)
        END = self._scan('END', context=_context)
        return parser.Source(toplevel)

    def function(self, _parent=None):
        _context = self.Context(_parent, self._scanner, 'function', [])
        vtype = self.vtype(_context)
        rtype = vtype
        ID = self._scan('ID', context=_context)
        fname = ID
        self._scan('"\\("', context=_context)
        args = []
        if self._peek('"\\)"', '"void"', '"byte"', '"array"', '","', context=_context) not in ['"\\)"', '","']:
            inout = False
            vtype = self.vtype(_context)
            if self._peek('"&"', 'ID', context=_context) == '"&"':
                self._scan('"&"', context=_context)
                inout = True
            ID = self._scan('ID', context=_context)
            args.append(parser.Argument(vtype, ID, inout))
            while self._peek('","', '"\\)"', context=_context) == '","':
                inout = False
                self._scan('","', context=_context)
                vtype = self.vtype(_context)
                if self._peek('"&"', 'ID', context=_context) == '"&"':
                    self._scan('"&"', context=_context)
                    inout = True
                ID = self._scan('ID', context=_context)
                args.append(parser.Argument(vtype, ID, inout))
        self._scan('"\\)"', context=_context)
        codeblock = self.codeblock(_context)
        return parser.Function(fname, args, rtype, codeblock)

    def vtype(self, _parent=None):
        _context = self.Context(_parent, self._scanner, 'vtype', [])
        _token = self._peek('"void"', '"byte"', '"array"', context=_context)
        if _token == '"void"':
            self._scan('"void"', context=_context)
            return parser.VoidType()
        elif _token == '"byte"':
            self._scan('"byte"', context=_context)
            return parser.ByteType()
        else: # == '"array"'
            self._scan('"array"', context=_context)
            self._scan('"\\["', context=_context)
            NUM = self._scan('NUM', context=_context)
            self._scan('"\\]"', context=_context)
            self._scan('"of"', context=_context)
            vtype = self.vtype(_context)
            return parser.ArrayType(string.atoi(NUM), vtype)

    def codeblock(self, _parent=None):
        _context = self.Context(_parent, self._scanner, 'codeblock', [])
        self._scan('"{"', context=_context)
        statements = []
        while self._peek('"}"', '"do"', '"{"', '"return"', '"def"', '"if"', '"while"', '"out"', '"\\$"', context=_context) != '"}"':
            statement = self.statement(_context)
            self._scan('";"', context=_context)
            statements.append(statement)
        self._scan('"}"', context=_context)
        return parser.CodeBlock(statements)

    def statement(self, _parent=None):
        _context = self.Context(_parent, self._scanner, 'statement', [])
        _token = self._peek('"do"', '"{"', '"return"', '"def"', '"if"', '"while"', '"out"', '"\\$"', context=_context)
        if _token == '"def"':
            variabledef = self.variabledef(_context)
            return variabledef
        elif _token == '"do"':
            self._scan('"do"', context=_context)
            expression = self.expression(_context)
            return parser.Execute(expression)
        elif _token == '"{"':
            codeblock = self.codeblock(_context)
            return codeblock
        elif _token == '"if"':
            ifelse = self.ifelse(_context)
            return ifelse
        elif _token == '"while"':
            whileloop = self.whileloop(_context)
            return whileloop
        elif _token != '"return"':
            assignment = self.assignment(_context)
            return assignment
        else: # == '"return"'
            self._scan('"return"', context=_context)
            return parser.Return()

    def variabledef(self, _parent=None):
        _context = self.Context(_parent, self._scanner, 'variabledef', [])
        self._scan('"def"', context=_context)
        vtype = self.vtype(_context)
        ID = self._scan('ID', context=_context)
        ids = [ID]
        while self._peek('","', '";"', '"else"', context=_context) == '","':
            self._scan('","', context=_context)
            ID = self._scan('ID', context=_context)
            ids.append(ID)
        return parser.VariableDef(vtype, ids)

    def assignment(self, _parent=None):
        _context = self.Context(_parent, self._scanner, 'assignment', [])
        _token = self._peek('"out"', '"\\$"', context=_context)
        if _token == '"\\$"':
            variable = self.variable(_context)
            _token = self._peek('"\\["', '"<-"', context=_context)
            if _token == '"\\["':
                self._scan('"\\["', context=_context)
                expression = self.expression(_context)
                self._scan('"\\]"', context=_context)
                lhs = parser.Index([variable, expression])
            else: # == '"<-"'
                lhs = variable
        else: # == '"out"'
            self._scan('"out"', context=_context)
            lhs = parser.Output()
        self._scan('"<-"', context=_context)
        expression = self.expression(_context)
        return parser.Assignment(lhs, expression)

    def ifelse(self, _parent=None):
        _context = self.Context(_parent, self._scanner, 'ifelse', [])
        truest = falsest = None
        self._scan('"if"', context=_context)
        self._scan('"\\("', context=_context)
        expression = self.expression(_context)
        self._scan('"\\)"', context=_context)
        condition = expression
        statement = self.statement(_context)
        truest = statement
        if self._peek('"else"', '";"', context=_context) == '"else"':
            self._scan('"else"', context=_context)
            statement = self.statement(_context)
            falsest = statement
        return parser.IfElse(condition, truest, falsest)

    def whileloop(self, _parent=None):
        _context = self.Context(_parent, self._scanner, 'whileloop', [])
        self._scan('"while"', context=_context)
        self._scan('"\\("', context=_context)
        expression = self.expression(_context)
        self._scan('"\\)"', context=_context)
        statement = self.statement(_context)
        return parser.While(expression, statement)

    def expression(self, _parent=None):
        _context = self.Context(_parent, self._scanner, 'expression', [])
        or_expr = self.or_expr(_context)
        return or_expr

    def literal(self, _parent=None):
        _context = self.Context(_parent, self._scanner, 'literal', [])
        _token = self._peek('"null"', 'NUM', '"\\""', context=_context)
        if _token == '"null"':
            self._scan('"null"', context=_context)
            return parser.NoneLiteral()
        elif _token == 'NUM':
            NUM = self._scan('NUM', context=_context)
            return parser.ByteLiteral(string.atoi(NUM))
        else: # == '"\\""'
            self._scan('"\\""', context=_context)
            TEXT = self._scan('TEXT', context=_context)
            self._scan('"\\""', context=_context)
            return parser.ByteArrayLiteral(eval("'"+TEXT+"'"))

    def funcall(self, _parent=None):
        _context = self.Context(_parent, self._scanner, 'funcall', [])
        args = []
        ID = self._scan('ID', context=_context)
        self._scan('"\\("', context=_context)
        fname = ID
        if self._peek('"\\)"', '","', '"in"', '"null"', 'NUM', '"\\""', 'ID', '"\\("', '"\\$"', context=_context) not in ['"\\)"', '","']:
            expression = self.expression(_context)
            args.append(expression)
            while self._peek('","', '"\\)"', context=_context) == '","':
                self._scan('","', context=_context)
                expression = self.expression(_context)
                args.append(expression)
        self._scan('"\\)"', context=_context)
        return parser.FunctionCall(fname, args)

    def or_expr(self, _parent=None):
        _context = self.Context(_parent, self._scanner, 'or_expr', [])
        and_expr = self.and_expr(_context)
        ret = and_expr
        while self._peek('"||"', '"\\]"', '"\\)"', '","', '";"', '"else"', context=_context) == '"||"':
            self._scan('"||"', context=_context)
            and_expr = self.and_expr(_context)
            ret = parser.BoolOr([ret, and_expr])
        return ret

    def and_expr(self, _parent=None):
        _context = self.Context(_parent, self._scanner, 'and_expr', [])
        eq_expr = self.eq_expr(_context)
        ret = eq_expr
        while self._peek('"&&"', '"||"', '"\\]"', '"\\)"', '","', '";"', '"else"', context=_context) == '"&&"':
            self._scan('"&&"', context=_context)
            eq_expr = self.eq_expr(_context)
            ret = parser.BoolAnd([ret, eq_expr])
        return ret

    def eq_expr(self, _parent=None):
        _context = self.Context(_parent, self._scanner, 'eq_expr', [])
        rel_expr = self.rel_expr(_context)
        ret = rel_expr
        while self._peek('"="', '"!="', '"&&"', '"||"', '"\\]"', '"\\)"', '","', '";"', '"else"', context=_context) in ['"="', '"!="']:
            _token = self._peek('"="', '"!="', context=_context)
            if _token == '"="':
                self._scan('"="', context=_context)
                rel_expr = self.rel_expr(_context)
                ret = parser.Equal([ret, rel_expr])
            else: # == '"!="'
                self._scan('"!="', context=_context)
                rel_expr = self.rel_expr(_context)
                ret = parser.Inequal([ret, rel_expr])
        return ret

    def rel_expr(self, _parent=None):
        _context = self.Context(_parent, self._scanner, 'rel_expr', [])
        sum_expr = self.sum_expr(_context)
        ret = sum_expr
        while self._peek('">"', '">="', '"<"', '"<="', '"="', '"!="', '"&&"', '"||"', '"\\]"', '"\\)"', '","', '";"', '"else"', context=_context) in ['">"', '">="', '"<"', '"<="']:
            _token = self._peek('">"', '">="', '"<"', '"<="', context=_context)
            if _token == '">"':
                self._scan('">"', context=_context)
                sum_expr = self.sum_expr(_context)
                ret = parser.MoreThan([ret, sum_expr])
            elif _token == '">="':
                self._scan('">="', context=_context)
                sum_expr = self.sum_expr(_context)
                ret = parser.MoreEqual([ret, sum_expr])
            elif _token == '"<"':
                self._scan('"<"', context=_context)
                sum_expr = self.sum_expr(_context)
                ret = parser.LessThan([ret, sum_expr])
            else: # == '"<="'
                self._scan('"<="', context=_context)
                sum_expr = self.sum_expr(_context)
                ret = parser.LessEqual([ret, sum_expr])
        return ret

    def sum_expr(self, _parent=None):
        _context = self.Context(_parent, self._scanner, 'sum_expr', [])
        factor_expr = self.factor_expr(_context)
        ret = factor_expr
        while self._peek('"\\+"', '"-"', '">"', '">="', '"<"', '"<="', '"="', '"!="', '"&&"', '"||"', '"\\]"', '"\\)"', '","', '";"', '"else"', context=_context) in ['"\\+"', '"-"']:
            _token = self._peek('"\\+"', '"-"', context=_context)
            if _token == '"\\+"':
                self._scan('"\\+"', context=_context)
                factor_expr = self.factor_expr(_context)
                ret = parser.Sum([ret, factor_expr])
            else: # == '"-"'
                self._scan('"-"', context=_context)
                factor_expr = self.factor_expr(_context)
                ret = parser.Difference([ret, factor_expr])
        return ret

    def factor_expr(self, _parent=None):
        _context = self.Context(_parent, self._scanner, 'factor_expr', [])
        postfix_expr = self.postfix_expr(_context)
        ret = postfix_expr
        while self._peek('"\\*"', '"/"', '"%"', '"\\+"', '"-"', '">"', '">="', '"<"', '"<="', '"="', '"!="', '"&&"', '"||"', '"\\]"', '"\\)"', '","', '";"', '"else"', context=_context) in ['"\\*"', '"/"', '"%"']:
            _token = self._peek('"\\*"', '"/"', '"%"', context=_context)
            if _token == '"\\*"':
                self._scan('"\\*"', context=_context)
                postfix_expr = self.postfix_expr(_context)
                ret = parser.Product([ret, postfix_expr])
            elif _token == '"/"':
                self._scan('"/"', context=_context)
                postfix_expr = self.postfix_expr(_context)
                ret = parser.Quotient([ret, postfix_expr])
            else: # == '"%"'
                self._scan('"%"', context=_context)
                postfix_expr = self.postfix_expr(_context)
                ret = parser.Modulo([ret, postfix_expr])
        return ret

    def postfix_expr(self, _parent=None):
        _context = self.Context(_parent, self._scanner, 'postfix_expr', [])
        term_expr = self.term_expr(_context)
        ret = term_expr
        while self._peek('"\\["', '"\\*"', '"/"', '"%"', '"\\+"', '"-"', '">"', '">="', '"<"', '"<="', '"="', '"!="', '"&&"', '"||"', '"\\]"', '"\\)"', '","', '";"', '"else"', context=_context) == '"\\["':
            self._scan('"\\["', context=_context)
            expression = self.expression(_context)
            self._scan('"\\]"', context=_context)
            ret = parser.Index([ret, expression])
        return ret

    def term_expr(self, _parent=None):
        _context = self.Context(_parent, self._scanner, 'term_expr', [])
        _token = self._peek('"in"', '"null"', 'NUM', '"\\""', 'ID', '"\\("', '"\\$"', context=_context)
        if _token == '"in"':
            self._scan('"in"', context=_context)
            return parser.Input()
        elif _token not in ['ID', '"\\("', '"\\$"']:
            literal = self.literal(_context)
            return literal
        elif _token == 'ID':
            funcall = self.funcall(_context)
            return funcall
        elif _token == '"\\$"':
            variable = self.variable(_context)
            return variable
        else: # == '"\\("'
            self._scan('"\\("', context=_context)
            expression = self.expression(_context)
            self._scan('"\\)"', context=_context)
            return expression

    def variable(self, _parent=None):
        _context = self.Context(_parent, self._scanner, 'variable', [])
        self._scan('"\\$"', context=_context)
        ID = self._scan('ID', context=_context)
        return parser.Variable(ID)


def parse(rule, text):
    P = HLBF(HLBFScanner(text))
    return runtime.wrap_error_reporter(P, rule)

if __name__ == '__main__':
    from sys import argv, stdin
    if len(argv) >= 2:
        if len(argv) >= 3:
            f = open(argv[2],'r')
        else:
            f = stdin
        print parse(argv[1], f.read())
    else: print >>sys.stderr, 'Args:  <rule> [<filename>]'
# End -- grammar generated by Yapps
