# High-Level Brainfuck
#
# Copyright (c) 2004 Marek Materzok
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.

class Node(object):
	def __init__(self):
		pass

class Source(Node):
	def __init__(self, ch = []):
		Node.__init__(self)
		self.children = ch

class Function(Node):
	def __init__(self, name, arglist, rettype, body):
		Node.__init__(self)
		self.name = name
		self.argument_list = arglist
		self.return_type = rettype
		self.body = body

class Argument(Node):
	def __init__(self, type, name, inout):
		Node.__init__(self)
		self.type = type
		self.name = name
		self.inout = inout

class Type(Node):
	pass

class VoidType(Type):
	pass

class ByteType(Type):
	pass

class ArrayType(Type):
	def __init__(self, len, type):
		Type.__init__(self)
		self.type = type
		self.len = len

class Statement(Node):
	pass

class CodeBlock(Statement):
	def __init__(self, ch = []):
		Statement.__init__(self)
		self.children = ch

class VariableDef(Statement):
	def __init__(self, type, names):
		Statement.__init__(self)
		self.names = names
		self.type = type

class Assignment(Statement):
	def __init__(self, lhs, rhs):
		Statement.__init__(self)
		self.lhs = lhs
		self.rhs = rhs

class Execute(Statement):
	def __init__(self, expr):
		Statement.__init__(self)
		self.expression = expr

class IfElse(Statement):
	def __init__(self, cond, ctrue, cfalse = None):
		Statement.__init__(self)
		self.condition = cond
		self.code_true = ctrue
		self.code_false = cfalse

class While(Statement):
	def __init__(self, cond, body):
		Statement.__init__(self)
		self.condition = cond
		self.body = body

class Return(Statement):
	pass

class Expression(Node):
	pass

class Input(Expression):
	pass

class Output(Node):
	pass

class Variable(Expression):
	def __init__(self, name):
		Expression.__init__(self)
		self.name = name

class Literal(Expression):
	def __init__(self, type, value):
		Expression.__init__(self)
		self.type = type
		self.value = value

class VoidLiteral(Literal):
	def __init__(self):
		Literal.__init__(self, VoidType(), None)

class ByteLiteral(Literal):
	def __init__(self, val):
		Literal.__init__(self, ByteType(), val)

class ByteArrayLiteral(Literal):
	def __init__(self, val):
		val += "\x00"
		Literal.__init__(self, ArrayType(len(val), ByteType()), val)

class ListExpression(Expression):
	def __init__(self, ch = []):
		Expression.__init__(self)
		self.children = ch

# suma
class Sum(ListExpression):
	pass

# roznica
class Difference(ListExpression):
	pass

# iloczyn
class Product(ListExpression):
	pass

# iloraz
class Quotient(ListExpression):
	pass

class Modulo(ListExpression):
	pass

class BoolAnd(ListExpression):
	pass

class BoolOr(ListExpression):
	pass

class LessThan(ListExpression):
	pass

class LessEqual(ListExpression):
	pass

class MoreThan(ListExpression):
	pass

class MoreEqual(ListExpression):
	pass

class Equal(ListExpression):
	pass

class Inequal(ListExpression):
	pass

class Index(ListExpression):
	pass

class FunctionCall(Expression):
	def __init__(self, name, args):
		Expression.__init__(self)
		self.name = name
		self.arguments = args

